import { world, system, Dimension, Vector3 } from "@minecraft/server"
import { pistonDirection } from "../interfaces"
import { apiInfo } from "../lib/apiInfo"

world.afterEvents.playerBreakBlock.subscribe(({brokenBlockPermutation: blockPerm, block: b, dimension}) => {
    if(blockPerm.type.id.includes("ws:waystone_")){
        if(!blockPerm.getState("ws:waystone_on")) return
        const block = blockPerm.getState("ws:waystone") == 1 ? b : b.below(1)
        removeWaystone(dimension, JSON.stringify(block.location))
    }
})

world.afterEvents.blockExplode.subscribe(({block: b, explodedBlockPermutation: blockPerm, dimension}) => {
    if(blockPerm.type.id.includes("ws:waystone_")){
        if(!blockPerm.getState("ws:waystone_on")) return
        const block = blockPerm.getState("ws:waystone") == 1 ? b : b.below(1)
        removeWaystone(dimension, JSON.stringify(block.location))
    }
})

const pistonDirection: pistonDirection[] = [["y", -1], ["y", 1], ["z", 1], ["z", -1], ["x", 1], ["x", -1]]

world.afterEvents.pistonActivate.subscribe(({block, dimension, isExpanding, piston}) => {
    const increase = pistonDirection[block.permutation.getState("facing_direction") as number]
    const locations = piston.getAttachedBlocksLocations()
    system.runTimeout(() => {
        for(let pos of locations){
            try{
                pos[increase[0]] += increase[1] * (isExpanding ? 1 : -1)
                const block = dimension.getBlock(pos)
                if(!block.typeId.includes("ws:waystone_")) continue
                dimension.setBlockType(pos, "minecraft:air")
                pos[increase[0]] += increase[1] * (isExpanding ? -1 : 1)
                removeWaystone(dimension, JSON.stringify(pos))
            } catch (e) { console.warn(e) }
        }
    }, 2)
})

function removeWaystone(dimension: Dimension, pos: string){
    const publicWay = apiInfo.getAllWaystones(null, "public")
    const waystonePub = publicWay.findIndex(way => JSON.stringify(way.pos) == pos && way.world == dimension.id)
    if(waystonePub != -1){
        publicWay.splice(waystonePub, 1)
        return apiInfo.saveAllWaystones(null, publicWay, "public")
    }
    const allWaystonePrivate = apiInfo.getAllPrivatePoints()
    for(const id of Object.keys(allWaystonePrivate)){
        const waystones = allWaystonePrivate[id]
        const waystoneIndex = waystones.findIndex(way => JSON.stringify(way.pos) == pos && way.world == dimension.id)
        if(waystoneIndex == -1) continue
        waystones.splice(waystoneIndex, 1)
        apiInfo.saveAllWaystones(id, waystones, "private")
    }
    const claimList: string[] = world.getDynamicPropertyIds().map(dyn => dyn.startsWith(`ws:${pos.replaceAll("\"", "'")}=`) ? dyn.replaceAll("\"", "'") : null).filter(value => value !== null)
    claimList.forEach(way => { world.setDynamicProperty(way, JSON.stringify([])) })
}